/****************************************************
* COKE MACHINE SIMULATION WITH AUTOMATA THEORY      *
* LINGUAGENS FORMAIS E AUTOMATOS                    *
* Prof.: Jones Oliveira Albuquerque                 *
*                                                   *
* INTEGRANTES:                                      *
* Anderson de Rezende Rocha [undersun@comp.ufla.br] *
* Flvio Luiz Alves         [alves@comp.ufla.br]    *
* Jlio Csar Alves         [jcalves@comp.ufla.br]  *
* Rodrigo Nazar Leite      [canniggia@comp.ufla.br]*
*                                                   *
* Bacharelado em Cincia da Computao              *
* Universidade Federal de Lavras, MG, Brasil        *
****************************************************/

// Define a classe que representa uma ligacao entre dois estados (uma producao)

#ifndef LIGACAO_H
#define LIGACAO_H

// OBSERVACAO
// A classe TSimbolo deve definir o operador ==
#include "estado.h"

// Diretiva para o compilador saber que futuramente encontrara tal classe
template <class TSimbolo>
class clEstado;

// Definicao da classe template clLigacao
template <class TSimbolo>
class clLigacao
{
    public:
        // Construtor a partir de um simbolo e um estado destino
        clLigacao<TSimbolo>(TSimbolo simbolo, clEstado<TSimbolo>* destino = NULL);
        // Construtor de copia
        clLigacao<TSimbolo>(const clLigacao<TSimbolo> &ligacao);
        // Apenas define o destrutor
        ~clLigacao<TSimbolo>() {};
        // Retorna o destino para o qual esta ligacao esta apontando
        clEstado<TSimbolo> *getDestino();
        // Retorna o simbolo usado nesta ligacao (simbolo consumido na producao)
        TSimbolo *getSimbolo();
        // Compara uma ligacao com outra somente em relacao ao simbolo
        bool operator == (clLigacao<TSimbolo> ligacao);

    private:
        // Guarda uma referencia ao destino desta ligacao
        clEstado<TSimbolo> *destino;
        // Simbolo desta producao
        TSimbolo simbolo;
};

// Construtor a partir de um simbolo e um estado destino
template <class TSimbolo>
clLigacao<TSimbolo>::clLigacao<TSimbolo>(TSimbolo simbolo, clEstado<TSimbolo>* destino)
{
    clLigacao<TSimbolo>::simbolo = simbolo;
    clLigacao<TSimbolo>::destino = destino;
}

// Construtor de copia
template <class TSimbolo>
clLigacao<TSimbolo>::clLigacao<TSimbolo>(const clLigacao<TSimbolo> &ligacao)
{
    simbolo = ligacao.simbolo;
    destino = ligacao.destino;
}

// Retorna o destino desta ligacao
template <class TSimbolo>
clEstado<TSimbolo> *clLigacao<TSimbolo>::getDestino()
{
    return destino;
}

// Retorna o simbolo desta ligacao
template <class TSimbolo>
TSimbolo *clLigacao<TSimbolo>::getSimbolo()
{
    return &simbolo;
}

// Duas ligacoes sao iguais de acordo com o seus simbolos
// Por Exemplo:
// se duas ligacoes tem simbolo 'a' e destinos q1 e q3, elas sao consideradas iguais
template <class TSimbolo>
bool clLigacao<TSimbolo>::operator == (clLigacao<TSimbolo> ligacao)
{
    return (*getSimbolo() == *(ligacao.getSimbolo()));
}

#endif

